//---------------------------------------------------------------------------
#pragma once

# pragma warning(push)
# pragma warning(disable:4355) // base class just stores the derived pointer
# pragma warning(disable:4312) // conversion uint to ptr
# pragma warning(disable:4275) // STL export
# pragma warning(disable:4251) // STL export

#include <string>
#include <vector>
#include <map>
#include "member_ptr.hpp"
#include "ExcelBinary.h"
//---------------------------------------------------------------------------
class ExcelFont;
class ExcelFormat;
class ExcelInterior;
class ExcelPtr;
class ExcelObject;
class ExcelCell;
class ExcelRange;
class ExcelWorksheet;
class ExcelWorksheets;
class ExcelWorkbook;
class ExcelWorkbooks;
class ExcelApplication;

typedef member_ptr<ExcelFont       >    FontPtr;
typedef member_ptr<ExcelFormat     >    FormatPtr;
typedef member_ptr<ExcelInterior   >    InteriorPtr;
typedef member_ptr<ExcelBinary::XF >    XFPtr;
typedef member_ptr<ExcelObject     >    ObjectPtr;
typedef member_ptr<ExcelCell       >    CellPtr;
typedef member_ptr<ExcelRange      >    RangePtr;
typedef member_ptr<ExcelWorksheet  >    WorksheetPtr;
typedef member_ptr<ExcelWorksheets >    WorksheetsPtr;
typedef member_ptr<ExcelWorkbook   >    WorkbookPtr;
typedef member_ptr<ExcelWorkbooks  >    WorkbooksPtr;
typedef member_ptr<ExcelApplication>    ApplicationPtr;
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
class 
		#ifdef __BUILD_DirectXLS
			__declspec( dllexport )
		#else
			__declspec( dllimport )
		#endif
ExcelFont
{
public:
    ExcelFont( WORD wSize, ExcelBinary::FontStyles eFontStyles, const char* lpszName)
	:   m_binaryFont( wSize, eFontStyles, lpszName)
	{}
    ExcelFont()
	{}
    ExcelFont( const ExcelFont& font)
	:   m_binaryFont( font.m_binaryFont)
	{}
    virtual ~ExcelFont(){}

    ExcelFont& operator=( const ExcelFont& font)
    {
		m_binaryFont = font.m_binaryFont;
        return *this;
    }
    bool operator==( const ExcelFont& font) const    { return (m_binaryFont == font.m_binaryFont); }
    bool operator!=( const ExcelFont& font) const    { return !operator==( font); }

public:
	void SetSize( WORD wSize)				{ m_binaryFont.SetSize( wSize);}
	WORD GetSize() const					{ return m_binaryFont.GetSize();}
	double GetHeight( bool fWithLineGap)	{ return m_binaryFont.GetHeight( fWithLineGap);}

	void SetBold( bool fIsBold)				{ m_binaryFont.SetBold( fIsBold);}
	bool IsBold() const						{ return m_binaryFont.IsBold();}
	void SetUnderline( bool fIsUnderline)	{ m_binaryFont.SetUnderline( fIsUnderline);}
	bool IsUnderline() const				{ return m_binaryFont.IsUnderline();}
	void SetItalic( bool fIsItalic)			{ m_binaryFont.SetItalic( fIsItalic);}
	bool IsItalic() const					{ return m_binaryFont.IsItalic();}
	void SetName( const char* lpszName)		{ m_binaryFont.SetName( lpszName);}
	const char* GetName() const				{ return m_binaryFont.GetName();}

	void SetColor( COLORREF refColor)					{ m_binaryFont.SetColor( refColor);}
	void SetColor( BYTE bRed, BYTE bGreen, BYTE bBlue)	{ m_binaryFont.SetColor( bRed, bGreen, bBlue);}

public:
	// for internal use of this framework only
	ExcelBinary::FONT&	Internal_GetBinaryFont()	{ return m_binaryFont;}

protected:
	ExcelBinary::FONT	m_binaryFont;
};
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
class 
		#ifdef __BUILD_DirectXLS
			__declspec( dllexport )
		#else
			__declspec( dllimport )
		#endif
ExcelFormat
{
public:
    ExcelFormat()
	{}
    ExcelFormat( const ExcelFormat& Format)
	:	m_binaryFormat( Format.m_binaryFormat)
	{}
    virtual ~ExcelFormat(){}

    ExcelFormat& operator=( const ExcelFormat& Format)
    {
        m_binaryFormat = Format.m_binaryFormat;
        return *this;
    }
    bool operator==( const ExcelFormat& format) const    { return (m_binaryFormat == format.m_binaryFormat); }
    bool operator!=( const ExcelFormat& format) const    { return !operator==( format); }

public:
	void SetFormatString( const char* lpszString)	{ m_binaryFormat.SetFormatString( lpszString);}
	const char*  GetFormatString()					{ return m_binaryFormat.GetFormatString();}
	const WCHAR* GetFormatStringW()					{ return m_binaryFormat.GetFormatStringW();}

public:
	// for internal use of this framework only
	ExcelBinary::FORMAT&	Internal_GetBinaryFormat()	{ return m_binaryFormat;}

protected:
	ExcelBinary::FORMAT		m_binaryFormat;
};
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
class 
		#ifdef __BUILD_DirectXLS
			__declspec( dllexport )
		#else
			__declspec( dllimport )
		#endif
ExcelInterior
{
public:
    enum Align_Horz {
                        ALIGN_HORZ_GENERAL  = ExcelBinary::ALIGN_HORZ_GENERAL,
                        ALIGN_HORZ_LEFT     = ExcelBinary::ALIGN_HORZ_LEFT,
                        ALIGN_HORZ_CENTER   = ExcelBinary::ALIGN_HORZ_CENTER,
                        ALIGN_HORZ_RIGHT    = ExcelBinary::ALIGN_HORZ_RIGHT,
                        ALIGN_HORZ_FILLED   = ExcelBinary::ALIGN_HORZ_FILLED,
                        ALIGN_HORZ_JUSTIFIED= ExcelBinary::ALIGN_HORZ_JUSTIFIED
                    };
    enum Align_Vert {
                        ALIGN_VERT_TOP      = ExcelBinary::ALIGN_VERT_TOP,
                        ALIGN_VERT_CENTER   = ExcelBinary::ALIGN_VERT_CENTER,
                        ALIGN_VERT_BOTTOM   = ExcelBinary::ALIGN_VERT_BOTTOM,
                        ALIGN_VERT_JUSTIFIED= ExcelBinary::ALIGN_VERT_JUSTIFIED
                    };
    enum Line_Styles{
                        LINE_STYLES_NONE    = ExcelBinary::LINE_STYLES_NONE,
                        LINE_STYLES_THIN    = ExcelBinary::LINE_STYLES_THIN,
                        LINE_STYLES_MEDIUM  = ExcelBinary::LINE_STYLES_MEDIUM,
                        LINE_STYLES_THICK   = ExcelBinary::LINE_STYLES_THICK,
                        LINE_STYLES_DOUBLE  = ExcelBinary::LINE_STYLES_DOUBLE,
                    };

public:
    ExcelInterior()
	{}
    ExcelInterior( const ExcelInterior& Interior)
	:	m_binaryXF( Interior.m_binaryXF)
	{}
    virtual ~ExcelInterior(){}

    ExcelInterior& operator=( const ExcelInterior& Interior)
    {
        m_binaryXF = Interior.m_binaryXF;
        return *this;
    }
    bool operator==( const ExcelInterior& XF) const    { return (m_binaryXF == XF.m_binaryXF); }
    bool operator!=( const ExcelInterior& XF) const    { return !operator==( XF); }

public:
        void SetDefault()		{ m_binaryXF.SetDefault();}
            //---------------
		void SetAlignmentHorz( Align_Horz eAlign_Horz)	{ m_binaryXF.SetAlignmentHorz( (ExcelBinary::Align_Horz)eAlign_Horz);}
        void SetAlignmentVert( Align_Vert eAlign_Vert)	{ m_binaryXF.SetAlignmentVert( (ExcelBinary::Align_Vert)eAlign_Vert);}
        void SetWrapText( bool fToWrap)					{ m_binaryXF.SetWrapText( fToWrap);}

		ExcelInterior::Align_Horz	GetAlignmentHorz() const		{ return (ExcelInterior::Align_Horz)m_binaryXF.GetAlignmentHorz();}
        ExcelInterior::Align_Vert	GetAlignmentVert() const		{ return (ExcelInterior::Align_Vert)m_binaryXF.GetAlignmentVert();}
        bool GetWrapText() const				{ return m_binaryXF.GetWrapText();}
            //---------------
		void SetLineLeft   ( Line_Styles eStyle)	{ m_binaryXF.SetLineLeft	( (ExcelBinary::Line_Styles)eStyle);}
        void SetLineRight  ( Line_Styles eStyle)	{ m_binaryXF.SetLineRight	( (ExcelBinary::Line_Styles)eStyle);}
        void SetLineTop    ( Line_Styles eStyle)	{ m_binaryXF.SetLineTop		( (ExcelBinary::Line_Styles)eStyle);}
        void SetLineBottom ( Line_Styles eStyle)	{ m_binaryXF.SetLineBottom	( (ExcelBinary::Line_Styles)eStyle);}
        void SetLineArround( Line_Styles eStyle)	{ m_binaryXF.SetLineArround	( (ExcelBinary::Line_Styles)eStyle);}

		ExcelInterior::Line_Styles GetLineLeft() const		{ return (ExcelInterior::Line_Styles)m_binaryXF.GetLineLeft();}
        ExcelInterior::Line_Styles GetLineRight() const	{ return (ExcelInterior::Line_Styles)m_binaryXF.GetLineRight();}
        ExcelInterior::Line_Styles GetLineTop() const		{ return (ExcelInterior::Line_Styles)m_binaryXF.GetLineTop();}
        ExcelInterior::Line_Styles GetLineBottom() const	{ return (ExcelInterior::Line_Styles)m_binaryXF.GetLineBottom();}
            //---------------
        void SetBackgroundColor( COLORREF refColor)					{ m_binaryXF.SetBackgroundColor( refColor);}
		void SetBackgroundColor( BYTE bRed, BYTE bGreen, BYTE bBlue){ m_binaryXF.SetBackgroundColor( bRed, bGreen, bBlue);}
            //---------------
public:
	// for internal use of this framework only
	ExcelBinary::XF&	Internal_GetBinaryXF()	{ return m_binaryXF;}

protected:
	ExcelBinary::XF		m_binaryXF;
};
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
class 
		#ifdef __BUILD_DirectXLS
			__declspec( dllexport )
		#else
			__declspec( dllimport )
		#endif
ExcelPtr : public member_ptr_owner_base
{
public:
    ExcelPtr( member_ptr_owner_base* pOwner) : member_ptr_owner_base( pOwner){}
};
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
class 
		#ifdef __BUILD_DirectXLS
			__declspec( dllexport )
		#else
			__declspec( dllimport )
		#endif
ExcelObject : public ExcelPtr
{
friend class ExcelWorksheet;
friend class ExcelWorkbook;

public:
    ExcelObject( member_ptr_owner_base* pOwner) : ExcelPtr( pOwner){}
    ExcelObject( const ExcelObject& obj) : ExcelPtr( obj.get_owner()) {  *this = obj;}
    virtual ~ExcelObject(){}

    ExcelObject& operator=( const ExcelObject& obj);
    bool operator==( const ExcelObject& obj) const;
    bool operator!=( const ExcelObject& obj) const	{ return !(*this == obj);}

public:
    virtual FontPtr     Font();
    virtual FormatPtr   Format();
    virtual InteriorPtr Interior();

    virtual ApplicationPtr Application() const =0;

protected:
    virtual XFPtr XF();

protected:
    ExcelFont       m_Font;
    ExcelFormat     m_Format;
    ExcelInterior   m_XF;
};
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
enum ECELL_TYPE { CELL_TYPE_NONE, CELL_TYPE_NORMAL, CELL_TYPE_ROW, CELL_TYPE_COL, CELL_TYPE_SHEET};
//---------------------------------------------------------------------------
class 
		#ifdef __BUILD_DirectXLS
			__declspec( dllexport )
		#else
			__declspec( dllimport )
		#endif
ExcelCell : public ExcelObject
{
friend class ExcelRange;
friend class ExcelWorksheet;

enum EVALUE_TYPE { VALTYPE_NONE, VALTYPE_TEXT, VALTYPE_NUMBER, VALTYPE_FORMULA};

public:
    ExcelCell( const ExcelCell& cell) : ExcelObject( cell.get_owner()){  *this = cell;}
    virtual ~ExcelCell(){}

    ExcelCell& operator=( const ExcelCell& cell);

public:
    void SetText( const char* lpszText);
	void SetText( const std::string& sText)	{ SetText( sText.c_str());}
    const std::string& GetText();
    void SetNumber( double dbNumber);

    void SetRowHeight( double dbHeight);
    void SetColWidth( double dbWidth);
    double GetRowHeight();
    double GetColWidth();

    void SetHidden(bool fIsHidden);
    bool IsHidden();

    WorksheetPtr Parent() const;
    WorkbookPtr Workbook() const;
    virtual ApplicationPtr Application() const;
    CellPtr GetPtr() const;

protected:
    ExcelCell( WORD wRow, WORD wCol, ExcelWorksheet& aParent, member_ptr_owner_base* pOwner);

    void SetFormula( const char* lpszFormula, bool fIsArray, WORD wRangeRowFirst, WORD wRangeColFirst,
                                                             WORD wRangeRowLast,   WORD wRangeColLast   );
    void SetFormula( const std::string& sFormula, bool fIsArray, WORD wRangeRowFirst, WORD wRangeColFirst,
                                                             WORD wRangeRowLast,   WORD wRangeColLast   )
	{
		SetFormula( sFormula.c_str(), fIsArray, wRangeRowFirst, wRangeColFirst, wRangeRowLast,   wRangeColLast   );
	}

	void OnPrepareToSave( ExcelCell* pCellParent_Worksheet,
                          ExcelCell* pCellParent_Row,
                          ExcelCell* pCellParent_Col);
    void Save( IStream* pIStream);
    void SetParent( ExcelWorksheet& Parent){ m_pParent = &Parent;}
    ExcelWorksheet* ParentInt() const{ return m_pParent;}

protected:
    ExcelWorksheet* m_pParent;

    WORD m_wRow;
    WORD m_wCol;

    ECELL_TYPE m_Type;
    EVALUE_TYPE m_eValyeType;
    ExcelBinary::FORMULA m_FORMULA;
    ExcelBinary::NUMBER m_NUMBER;
    std::string m_sText;
    ExcelBinary::LABELSST m_LABELSST;

    double m_dbWidth;
    double m_dbHeight;
    bool m_fIsHidden;
};
//---------------------------------------------------------------------------
class 
		#ifdef __BUILD_DirectXLS
			__declspec( dllexport )
		#else
			__declspec( dllimport )
		#endif
ExcelRange :	public ExcelObject
					, protected ExcelBinary::CxNotifyConsumer
{
friend class ExcelWorksheet;

public:
    ExcelRange();
    ExcelRange( const ExcelRange& r);
    virtual ~ExcelRange(){}

    ExcelRange& operator=( const ExcelRange& r);
    bool operator==( const ExcelRange& r);
    bool operator!=( ExcelRange const& r);

public:
    bool isNull() const;

    void SetText( const char* lpszText);
	void SetText( const std::string& lpszText)	{ SetText( lpszText.c_str());}
    void SetNumber( double dbNumber);
    void SetFormula( const char* lpszFormula);
	void SetFormula( const std::string& sFormula)	{ SetFormula( sFormula.c_str());}
    void SetFormulaArray( const char* lpszFormula);
	void SetFormulaArray( const std::string& sFormula)	{ SetFormulaArray( sFormula.c_str());}
    const std::string& GetText();

    RangePtr Range(  int nRow, int nCol);
    RangePtr Expand( int nRows, int nCols);
    RangePtr GetEntireRows() const;
    RangePtr GetEntireCols() const;

    void SetMerged( bool fIsMerged);

    void SetColWidth( double dbWidth);
    double GetColWidth();
    double GetWidth();
    void SetRowHeight( double dbHeight);
    double GetRowHeight();
    double GetHeight();
    double GetDefaultRowHeight();

    double GetTextCalculatedHeight( const std::string& szText);
    double GetTextCalculatedHeight();
    void AutoFitLastRowHeight();

    void SetHidden(bool fIsHidden);
    bool IsHidden();

    WORD GetRow();
    WORD GetCol();
    std::string GetAddress( bool fLeftTop);
    DWORD GetCount();
    bool IsOneCell();

    WorksheetPtr Parent() const;
    virtual ApplicationPtr Application() const;
    RangePtr GetPtr() const;

protected:
    ExcelWorksheet* m_pExcelWorksheet;

    WORD m_wRowFirst;
    WORD m_wRowLast;
    WORD m_wColFirst;
    WORD m_wColLast;

    ECELL_TYPE m_Type;
    bool m_fIsMerged;

protected:
    ExcelRange( ExcelWorksheet& aExcelWorksheet, WORD wRowFirst,
                WORD wColFirst, WORD wRowLast, WORD wColLast);

	void Notify( ExcelBinary::CxNotifyHandler* caller)
	{
		if	   ( caller == &m_xNotifyHandler_OnFontChange)
			OnFontChange();
		else if( caller == &m_xNotifyHandler_OnFormatChange)
			OnFormatChange();
		else if( caller == &m_xNotifyHandler_OnInteriorChange)
			OnInteriorChange();
	}

protected:
	ExcelBinary::CxNotifyHandler	m_xNotifyHandler_OnFontChange;
	ExcelBinary::CxNotifyHandler	m_xNotifyHandler_OnFormatChange;
	ExcelBinary::CxNotifyHandler	m_xNotifyHandler_OnInteriorChange;
    void OnFontChange();
    void OnFormatChange();
    void OnInteriorChange();

protected:
	std::string NumToStr( DWORD dwNum);
    RangePtr CreateRange( WORD wRowFirst, WORD wColFirst, WORD wRowLast, WORD wColLast) const;
    ExcelWorksheet* ParentInt() const{ return m_pExcelWorksheet;}

private:
    void Init( ExcelWorksheet* pExcelWorksheet, WORD wRowFirst,
               WORD wColFirst, WORD wRowLast, WORD wColLast);
};
//---------------------------------------------------------------------------
typedef std::map<DWORD,ExcelCell*>  ONE_ROW;
typedef std::map<DWORD,ONE_ROW*>    TABLE;
//---------------------------------------------------------------------------
class 
		#ifdef __BUILD_DirectXLS
			__declspec( dllexport )
		#else
			__declspec( dllimport )
		#endif
ExcelWorksheet : public ExcelObject, protected TABLE
{
friend class ExcelCell;
friend class ExcelRange;
friend class ExcelWorksheets;
friend class ExcelWorkbook;

public:
    ExcelWorksheet( const ExcelWorksheet& ws) : ExcelObject( ws.get_owner()){  *this = ws;}
    virtual ~ExcelWorksheet();

    ExcelWorksheet& operator=( const ExcelWorksheet& ws);

public:
    void SetName( const char* sName);
	void SetName( const std::string& sName)	{ SetName( sName.c_str());}
	const std::string& GetName();

    void SetSelected();
    bool IsSelected();

    virtual FontPtr     Font();
    virtual FormatPtr   Format();
    virtual InteriorPtr Interior();

    RangePtr Range( WORD wRow, WORD wCol);
    RangePtr Range( WORD wRowFirst, WORD wColFirst,
                    WORD wRowLast, WORD wColLast);
    RangePtr Range( const char* szAddress);
	RangePtr Range( const std::string& szAddress)	{ return Range( szAddress.c_str());}

    RangePtr Cells();
    RangePtr Row( WORD wRow);
    RangePtr Col( WORD wCol);

    WORD GetRight();
    WORD GetRightMax();

    WorksheetsPtr Parent() const;
    WorkbookPtr Workbook() const;
    virtual ApplicationPtr Application() const;
    WorksheetPtr GetPtr() const;

protected:
    ExcelWorksheet( const char* szName, ExcelWorksheets* aParent, member_ptr_owner_base* pOwner);

	DWORD GetPos_Start();
    void SetParent( ExcelWorksheets* Parent);
    ExcelCell* Cell( WORD wRow, WORD wCol);
    ObjectPtr GetCellParentObject( WORD wRow, WORD wCol);
    FontPtr  GetCellFontToDisplay( WORD wRow, WORD wCol);
    void SetMergedRange( WORD wRowFirst, WORD wColFirst, WORD wRowLast, WORD wColLast);

    void OnPrepareToSave();
    void Save( IStream* pIStream);
    double GetRowHeightDefault( WORD wRow);

    ExcelWorksheets* ParentInt() const{ return m_pParent;}

protected:
    ExcelWorksheets* m_pParent;
    std::string m_sName;
    DWORD m_dwPos_Start;
    bool m_fIsSelected;
    ExcelBinary::MERGED_CELLS m_vecMerged;

    WORD m_wRight;

protected:
    void SetSelected( bool fSelected){ m_fIsSelected = fSelected;}
    ExcelCell* CreateCell( WORD wRow, WORD wCol);
    virtual XFPtr XF();
    RangePtr CreateRange( WORD wRowFirst, WORD wColFirst, WORD wRowLast, WORD wColLast);
};
//---------------------------------------------------------------------------
class 
		#ifdef __BUILD_DirectXLS
			__declspec( dllexport )
		#else
			__declspec( dllimport )
		#endif
ExcelWorksheets : protected std::vector<ExcelWorksheet*>, public ExcelObject
{
friend class ExcelCell;
friend class ExcelWorksheet;
friend class ExcelWorkbook;

typedef std::vector<WorksheetPtr> base_class;

public:
    ExcelWorksheets( const ExcelWorksheets& wss) : ExcelObject( wss.get_owner()){  *this = wss;}
    virtual ~ExcelWorksheets();

    ExcelWorksheets& operator=( const ExcelWorksheets& wbs);

public:
    WorksheetPtr Add( const char* szName);
	WorksheetPtr Add( const std::string& szName)	{ return Add( szName.c_str());}
    WorksheetPtr Add();

    DWORD Count();
    WorksheetPtr operator[]( DWORD dwIndex);

    WorkbookPtr Parent() const;
    WorkbookPtr Workbook() const;
    virtual ApplicationPtr Application() const;
    WorksheetsPtr GetPtr() const;

protected:
    ExcelWorksheets( member_ptr_owner_base* pOwner) : ExcelObject( pOwner){}
    void SetParent( ExcelWorkbook& Parent);
    void CheckIfNameUnique( const char* szName);
	void CheckIfNameUnique( const std::string& sName)	{ CheckIfNameUnique( sName.c_str());}
    VECTOR_CSTR GetNames();
    ExcelWorkbook* ParentInt() const{ return m_pParent;}

protected:
    void OnPrepareToSave();
    void Save( IStream* pIStream);

protected:
    ExcelWorkbook* m_pParent;
};
//---------------------------------------------------------------------------
class 
		#ifdef __BUILD_DirectXLS
			__declspec( dllexport )
		#else
			__declspec( dllimport )
		#endif
ExcelWorkbook : public ExcelObject
{
friend class ExcelCell;
friend class ExcelWorksheet;
friend class ExcelWorksheets;
friend class ExcelWorkbooks;

public:
    ExcelWorkbook( const ExcelWorkbook& wb) : ExcelObject( wb.get_owner()), m_Worksheets(this){    *this = wb;}
    virtual ~ExcelWorkbook(){}

    ExcelWorkbook& operator=( const ExcelWorkbook& wb);

public:
    void SetName( const char* szName);
	void SetName( const std::string& szName)	{ SetName( szName.c_str());}
    const std::string& GetName();

    WorksheetsPtr Worksheets();
    WorksheetPtr WorkSheet( DWORD dwIndex);

    WorkbooksPtr Parent() const;
    virtual ApplicationPtr Application() const;
    WorkbookPtr GetPtr() const;

protected:
    ExcelWorkbook( const char* szName, ExcelWorkbooks* aParent, member_ptr_owner_base* pOwner);
    void SetParent( ExcelWorkbooks* Parent);
    void FillSetXF( ExcelBinary::FONT& font, ExcelBinary::FORMAT& format, ExcelBinary::XF& xf,
                    ExcelCell* pCellParent_Worksheet,
                    ExcelCell* pCellParent_Row,
                    ExcelCell* pCellParent_Col);
    int SetText( const char* lpszString);
	int SetText( const std::string& sString)	{ return SetText( sString.c_str());}

    ExcelBinary::SUPBOOK* Supbook();

    void OnPrepareToSave();
    void Save( IStream* pIStream);
    ExcelWorkbooks* ParentInt() const{ return m_pParent;}

protected:
    ExcelWorkbooks* m_pParent;
    std::string m_sName;

    ExcelWorksheets m_Worksheets;

    typedef std::vector<ExcelBinary::FONT> typeVecFonts;
    typeVecFonts m_vecFonts;
    typedef std::vector<ExcelBinary::XF> typeVecXFs;
    std::vector<ExcelBinary::XF> m_vecXFs;
    typedef std::vector<ExcelBinary::FORMAT> typeVecFormats;
    typeVecFormats m_vecFormats;

    ExcelBinary::SST m_SST;
    ExcelBinary::SUPBOOK m_SUPBOOK;
    ExcelBinary::PALETTE m_Palette;

protected:
    WORD SetFont( ExcelBinary::FONT& font);
    int SetFormat( ExcelBinary::FORMAT& format);
};
//---------------------------------------------------------------------------
class 
		#ifdef __BUILD_DirectXLS
			__declspec( dllexport )
		#else
			__declspec( dllimport )
		#endif
ExcelWorkbooks : protected std::vector<ExcelWorkbook*>, public ExcelObject
{
friend class ExcelCell;
friend class ExcelWorksheet;
friend class ExcelWorkbook;
friend class ExcelApplication;

typedef std::vector<WorkbookPtr> base_class;

public:
    ExcelWorkbooks( const ExcelWorkbooks& wbs) : ExcelObject( wbs.get_owner()) {  *this = wbs;}
    virtual ~ExcelWorkbooks();

    ExcelWorkbooks& operator=( const ExcelWorkbooks& wbs);

public:
    WorkbookPtr Add( const char* szName);
	WorkbookPtr Add( const std::string& sName)	{ return Add( sName.c_str());}
    WorkbookPtr Add();

    DWORD Count();
    WorkbookPtr operator[]( DWORD dwIndex);

    ApplicationPtr Parent() const;
    virtual ApplicationPtr Application() const;
    WorkbooksPtr GetPtr() const;

protected:
    ExcelWorkbooks( member_ptr_owner_base* pOwner) : ExcelObject( pOwner){}
    void OnPrepareToSave();
    void Save( IStream* pIStream);
    void SetParent( ExcelApplication& Parent);
    void CheckIfNameUnique( const char* szName);
	void CheckIfNameUnique( const std::string& sName)	{ CheckIfNameUnique( sName.c_str());}
    ExcelApplication* ParentInt() const{ return m_pParent;}

protected:
    ExcelApplication* m_pParent;
};
//---------------------------------------------------------------------------
class 
		#ifdef __BUILD_DirectXLS
			__declspec( dllexport )
		#else
			__declspec( dllimport )
		#endif
ExcelApplication : public ExcelObject
{
friend class ExcelCell;
friend class ExcelRange;

public:
    ExcelApplication();
    virtual ~ExcelApplication()
    {}

public:
    void LoadFunctions();
    void QueryLocaleData();
    void QueryMonitorData();

    void Save( const char* szFilePath);
	void Save( const std::string& sFilePath)	{ Save( sFilePath.c_str());}

    WorkbooksPtr Workbooks();
    WorkbookPtr Workbook( DWORD dwIndex);

    char GetDecimalSeparator()	const;
    char GetThousandsSeparator()const;
    char GetListSeparator()		const;
    char GetColumnSeparator()	const;
    const std::string& GetGeneralFormatName();

    ApplicationPtr GetPtr() const;
    virtual ApplicationPtr Application() const;

protected:
    const ExcelBinary::FUNC_ARRAY& GetFuncArray();

    double GetDefaultWidth();
    double GetKoef_CharsToPix();
    double GetKoef_PointsToPix();

    void OnPrepareToSave();

protected:
    ExcelWorkbooks m_Workbooks;

    ExcelBinary::FUNC_ARRAY m_FuncArray;

    std::string m_sDecimalSeparator;
    std::string m_sThousandsSeparator;
    std::string m_sListSeparator;
    std::string m_sColumnSeparator;
    std::string m_sGeneralFormatName;

    double m_dbScreenYMill;
    double m_dbScreenYPix;

private:
    ExcelApplication( const ExcelApplication& excel) : ExcelObject( excel.get_owner()), m_Workbooks(this)  { *this = excel;}
	ExcelApplication& operator=( const ExcelApplication& wbs);
};
//---------------------------------------------------------------------------
# pragma warning(pop)

